<?php
namespace RealID\Hooks;

use RealID\ApiClient;
use Psr\Log\LoggerInterface;
use RealID\Models\IDCheck;
use RealID\Interfaces\Views;

class Orders {
  public $views;

  public function __construct(Views $views) {
    $this->views = $views;
  }

  public function register() {
    add_action( 'woocommerce_thankyou', array($this, 'post_checkout_order_script_data'), 10, 1 ); 
    add_action( 'woocommerce_thankyou', array($this, 'post_checkout_webhook'), 10, 1 ); 
    add_action( 'add_meta_boxes', [ $this, 'add_metabox' ] );
    add_filter( 'manage_edit-shop_order_columns', [$this, 'add_real_id_column']);
    add_action( 'manage_shop_order_posts_custom_column', [$this, 'populate_id_check_column'] );
    // woocommerce_add_to_cart is the actual event that fires when an item is added to the cart
    // but we need to render this cart JS object on all pages, not just when the product is added
    add_action( 'woocommerce_after_cart', [$this, 'render_cart_contents'], 10, 1);
  }

  public function post_checkout_order_script_data($order_get_id) {
    $order_data = real_id_get_order_data( $order_get_id );

    ?>
      <script type="text/javascript">
        window.realIdOrder = JSON.parse('<?php echo json_encode($order_data); ?>');
      </script>
    <?php
  }

  public function post_checkout_webhook($order_get_id) {
    // get order data
    $order_data = real_id_get_order_data($order_get_id);

    // pre-checkout
    if(real_id_is_pre_checkout_enabled()) {
      ob_start(); ?> 
      <script>
        console.log(`Real ID ::  post-checkout check <-> order association script`);
        const checkId = window.localStorage.getItem('real-id-check-id');
        const customerId = <?php echo (isset($order_data['customer']) ?  $order_data['customer']['ID'] : 'null'); ?>;
        const orderId = <?php echo $order_get_id; ?>;
        console.log(`Real ID :: post-checkout check <-> order association script :: found check ID ${checkId}`);
        // IF THIS ROUTE 404's
        // It is most likely because the wp instance has it's permalinks NOT set to "post", and instead they're using the defaults.
        jQuery.post(`/wp-json/real-id/v1/check/order/associate`, { checkId: checkId, orderId: orderId, customerId: customerId }, () => {console.log('sent')});
      </script> 
      
      <?php echo ob_get_clean();
    } 

    // get license key
    $licenseKey = real_id_get_license_key();
    $client = new ApiClient($licenseKey, [ 'http_errors' => true ]);
    
    // POST to Real API
    $response = $client->client->post('/api/webhooks/wc/orders', array('json' => $order_data));
  }

  /**
   * Add metabox to order details page to link back to Real ID check details
   * 
   */
  public function add_metabox() {
    add_meta_box(
        'realid_box_id',          // Unique ID
        'ID verification results', // Box title
        [ self::class, 'meta_box_html' ],   // Content callback, must be of type callable
        'shop_order' // Post type
    );
  }

    /**
     * Display the Real ID check status meta box HTML to the user within the order details page.
     *
     * @param \WP_Post $post   Post object.
     * @TODO React everywhereeeeee
     */
    public static function meta_box_html( $post ) {
        $check_id = get_post_meta($post->ID, 'real_id_check_id', true);

        if(is_string($check_id) && !empty($check_id)) {
          echo sprintf("<a class='button' href='/wp-admin/admin.php?page=real-id#/checks/%s'>View ID check results</a>", $check_id);
          return;
        }

        echo "<p>No ID check has been sent for this order.</p>";
        echo "<a class='button' href='/wp-admin/admin.php?page=real-id#/new'>Create ID Check</a>";
    }

  public function add_real_id_column($columns) {
    $column = ['real_id_check' => 'ID verification'];

    // split the columns so we can inject in the 2nd to last position
    $first = array_slice($columns, 0, count($columns) - 1, true);
    $last = array_slice($columns, -1, null);

    return $first + $column + $last;
  }

  /**
   * Populate the "ID verification" column in the WC Admin Orders page
   * 
   */
  public function populate_id_check_column($column_name) {
    global $the_order;

    if($column_name == 'real_id_check') {
      $current_status = $the_order->get_meta('real_id_check_status');
      $check_id = $the_order->get_meta('real_id_check_id');

      if(!empty($current_status) && isset($current_status)) {
        $this->views->render('admin/orders/id_verification_status_badge.html', [ 
          'status' => $current_status,
          'check_id' => $check_id,
        ]);
      }
    }
  }
  /**
   * Render and inject the cart contents into the frontend
   */
  public function render_cart_contents() {
    $cart = WC()->cart->get_cart();

    $line_items = array_values(array_map(function ($item) {
      return [
        'product_id' => $item['product_id'],
        'category_ids' => $item['data']->get_category_ids(),
      ];
    }, $cart));

    $category_ids = array_reduce($line_items, function($carry, $item) {
      $category_ids = $item['category_ids'];
      return array_merge($category_ids, $carry);
    }, []);

    $content = [
      'total' => (float) WC()->cart->total,
      'line_items' => $line_items,
      'category_ids' => $category_ids,
    ]
    
    ?>
      <script type="text/javascript">
        window.realIdCart = JSON.parse('<?php echo json_encode($content) ?>');
      </script>
    <?php
  }
}
