import { Page, Layout, ContextualSaveBar, Tabs, Icon } from "@shopify/polaris";
import React, { useEffect, useState, useContext } from "react";
import { Formik } from "formik";
import get from "lodash/get";
import { useToast } from "providers/ToastProvider";
import { useShop } from "providers/ShopProvider";
import {
  defaultCheckContent,
  defaultImagePublicId,
  defaultPrimaryColor,
} from "../constants";
import { Flex, Box } from "rebass";
import { updateSettings as callUpdateSettings } from "services/wpApi";
import { amplitudeEvent } from "components/initAmplitude";
import * as yup from "yup";
import AutomaticIdVerificationRules from "components/settings/AutomaticIdVerificationRules";
import IdCheckAppearance from "components/settings/IdCheckAppearance";
// import AdditionalCheckRequirements from "components/settings/AdditionalCheckRequirements";
import IdCheckNotificationsSettings from "components/settings/IdCheckNotificationSettings";
import TermsAndConditions from "components/settings/TermsAndConditions";
import Billing from "components/settings/Billing";
import EmailSenderSignatureSettings from "./settings/EmailSenderSignatureSettings";
import {
  FcSettings,
  FcClearFilters,
  FcEditImage,
  FcAbout,
  FcMoneyTransfer,
} from "react-icons/fc";
import DeactivateLicenseKey from "components/settings/DeactivateLicenseKey";

export default function Settings() {
  const setToast = useToast();
  const { shop, setShop } = useShop();
  console.log(shop);

  const [isLoading, setIsLoading] = useState(true);
  const [selectedTab, setSelectedTab] = useState(0);

  function updateSettings(values, setSubmitting) {
    setSubmitting(true);
    callUpdateSettings(values)
      .then(({ data }) => {
        setSubmitting(false);
        setShop(data);
      })
      .catch((err) => {
        Sentry.captureException(err);
        setToast(
          "Unable to update your shop settings. Please contact support."
        );
        setSubmitting(false);
      });
  }

  const tabs = [
    {
      id: "automatic-triggers",
      content: (
        //#637381
        <Flex alignItems="center">
          <FcSettings style={{ marginRight: "5px" }} />
          <span>Triggers</span>
        </Flex>
      ),
      accessibilityLabel: "Automatic ID triggers",
      panelID: "automatic-triggers-content-1",
    },
    {
      id: "appearance",
      content: (
        //#637381

        <Flex alignItems="center">
          <FcEditImage style={{ marginRight: "5px" }} />
          <span>Appearance</span>
        </Flex>
      ),
      accessibilityLabel: "ID check appearance",
      panelID: "Appearance",
    },
    // {
    //   id: "automatic-rules",
    //   content: (
    //     //#637381
    //     <Flex alignItems="center">
    //       <FcClearFilters style={{ marginRight: "5px" }} />
    //       <span>Rules</span>
    //     </Flex>
    //   ),
    //   accessibilityLabel: "Automatic ID rules",
    //   panelID: "automatic-rules-content-1",
    // },
    {
      id: "notifications",
      content: (
        //#637381

        <Flex alignItems="center">
          <FcAbout style={{ marginRight: "5px" }} />
          <span>Notifications</span>
        </Flex>
      ),
      accessibilityLabel: "Notifications on ID checks",
      panelID: "automatic-notifications-content-1",
    },
    {
      id: "billing",
      content: (
        //#637381
        <Flex alignItems="center">
          <FcMoneyTransfer style={{ marginRight: "5px" }} />
          <span>Billing</span>
        </Flex>
      ),
      accessibilityLabel: "Billing details",
      panelID: "automatic-billing-content-1",
    },
  ];

  return (
    <>
      <Page>
        <Tabs
          tabs={tabs}
          selected={selectedTab}
          onSelect={(tab) => setSelectedTab(tab)}
        >
          <Box pt={3}>
            <Formik
              enableReinitialize
              initialValues={{
                checkAllOrders: get(shop, "settings.checkAllOrders"),
                priceFloor: get(shop, "settings.priceFloor", 100),
                priceFloorEnabled: get(
                  shop,
                  "settings.priceFloorEnabled",
                  false
                ),
                shippingBillingAddressMismatchEnabled: get(
                  shop,
                  "settings.shippingBillingAddressMismatchEnabled",
                  false
                ),
                orderRiskLevelsEnabled: get(
                  shop,
                  "settings.orderRiskLevelsEnabled",
                  false
                ),
                orderRiskLevels: get(shop, "settings.orderRiskLevels", []),
                uniqueCustomersOnlyEnabled: get(
                  shop,
                  "settings.uniqueCustomersOnlyEnabled"
                ),
                defaultContent: get(
                  shop,
                  "settings.defaultContent",
                  defaultCheckContent
                ),
                minimumAge: get(shop, "settings.minimumAge", ""),
                minimumAgeEnabled: get(
                  shop,
                  "settings.minimumAgeEnabled",
                  false
                ),

                contactEmails: get(shop, "settings.contactEmails", [
                  get(shop, "settings.contactEmail", shop.contactEmail),
                ]),
                monitoredChannels: get(shop, "settings.monitoredChannels", []),
                crossCheckCustomerLastName: get(
                  shop,
                  "settings.crossCheckCustomerLastName"
                ),
                crossCheckShippingAddress: get(
                  shop,
                  "settings.crossCheckShippingAddress"
                ),
                checkOnlyUsa: get(shop, "settings.checkOnlyUsa", false),
                requiredIdUsStates: get(
                  shop,
                  "settings.requiredIdUsStates",
                  []
                ),
                usStateSpecificity: get(
                  shop,
                  "settings.usStateSpecificity",
                  "all_usa"
                ),
                primaryColor: get(
                  shop,
                  "settings.primaryColor",
                  defaultPrimaryColor
                ),
                idCheckType: get(shop, "settings.idCheckType", "idv"),
                signatureRequired: get(
                  shop,
                  "settings.signatureRequired",
                  false
                ),
                disableAutomaticNotes: get(
                  shop,
                  "settings.disableAutomaticNotes",
                  false
                ),
                imageUrl: get(shop, "settings.imageUrl", false),
                imagePublicId: get(
                  shop,
                  "settings.imagePublicId",
                  defaultImagePublicId
                ),
                requiredIdTagsEnabled: get(
                  shop,
                  "settings.requiredIdTagsEnabled"
                ),
                requiredIdCategoriesEnabled:
                  shop.settings?.requiredIdCategoriesEnabled,
                requiredIdCategories: get(
                  shop,
                  "settings.requiredIdCategories",
                  []
                ),
                requiredIdTagsEnabled: get(
                  shop,
                  "settings.requiredIdTagsEnabled"
                ),
                requiredIdTags: get(shop, "settings.requiredIdTags", []),
                preferredLocale: get(shop, "settings.preferredLocale", "en"),
                emailContent: {
                  customer: {
                    check: {
                      completed:
                        shop?.settings?.emailContent?.customer?.check
                          ?.completed ||
                        `Hello [firstName],

Thanks for completing ID verification!

We're reviewing the results and we'll be in touch about your shipment date & order details.

Thank you for helping keep our shop secure and protect against fraud.

If you have any questions, please contact email our customer support.
`,

                      in_review:
                        shop?.settings?.emailContent?.customer?.check
                          ?.in_review ||
                        `Hello [firstName],

We have received your ID for verification, but we were unable to automatically verify it.

A team member will be manually reviewing your case.

If you have any questions, please email our customer service team.

Thank you`,

                      failed:
                        shop?.settings?.emailContent?.customer?.check?.failed ||
                        `Hello [firstName],

Unfortunately after a manual review, we were not able to conclusively pass your ID.

Please contact customer support for the status of your refund if you have made a purchase.

If you have any questions or believe this is a mistake, please contact email our customer support.`,
                    },
                  },
                },
              }}
              validationSchema={yup.object().shape({
                priceFloor: yup
                  .number()
                  .min(0)
                  .when("priceFloorEnabled", {
                    is: true,
                    then: yup.number().required("A price is required"),
                  }),
                // minimumAge: yup
                // .string()
                // .matches(
                //   /(disabled|legal_canadian_smoking_ages|21|18|\d+)/,
                //   "A valid age is required"
                // ) // weird stuff with the regex based rule, it doesn't seem to work properly
                // .required(),
                // more weird stuff, an empty string causes this to break
                // contactEmails: yup
                //   .array()
                //   .of(
                //     yup.string().email().required("A valid email is required")
                //   ),
              })}
              onSubmit={(values, { setSubmitting }) => {
                updateSettings(values, setSubmitting);
              }}
            >
              {({
                values,
                errors,
                dirty,
                handleBlur,
                handleSubmit,
                isSubmitting,
                isValid,
                resetForm,
                setFieldValue,
              }) => {
                return (
                  <Layout>
                    {selectedTab === 0 && (
                      <AutomaticIdVerificationRules
                        values={values}
                        setFieldValue={setFieldValue}
                        handleBlur={handleBlur}
                        errors={errors}
                        shop={shop}
                      />
                    )}
                    {selectedTab === 1 && (
                      <IdCheckAppearance
                        values={values}
                        setFieldValue={setFieldValue}
                        handleBlur={handleBlur}
                        errors={errors}
                        shop={shop}
                      />
                    )}
                    {/*
                      {selectedTab === 2 && (
                        <AdditionalCheckRequirements
                          values={values}
                          setFieldValue={setFieldValue}
                          handleBlur={handleBlur}
                          errors={errors}
                          shop={shop}
                        />
                      )}
                    */}
                    {selectedTab === 2 && (
                      <>
                        <IdCheckNotificationsSettings
                          values={values}
                          setFieldValue={setFieldValue}
                          handleBlur={handleBlur}
                          errors={errors}
                          shop={shop}
                        />
                        <EmailSenderSignatureSettings
                          values={values}
                          setFieldValue={setFieldValue}
                          handleBlur={handleBlur}
                          errors={errors}
                          shop={shop}
                          setShop={setShop}
                        />
                      </>
                    )}
                    {/* {selectedTab === 4 && ( */}
                    {selectedTab === 3 && (
                      <>
                        <Billing shop={shop} />
                        {!shop.sandboxMode && shop.licenseKey && (
                          <DeactivateLicenseKey />
                        )}
                        <TermsAndConditions />
                      </>
                    )}
                    {dirty && (
                      <>
                        <style jsx>{`
                          .Polaris-Frame__ContextualSaveBar {
                            top: 32px;
                          }
                        `}</style>

                        <ContextualSaveBar
                          message={
                            isValid
                              ? "Unsaved changes"
                              : Object.values(errors)[0]
                          }
                          fullWidth
                          alignContentFlush={false}
                          discardAction={{
                            content: "Discard",
                            onAction: resetForm,
                          }}
                          saveAction={{
                            content: "Save",
                            onAction: handleSubmit,
                            loading: isSubmitting,
                            disabled: !isValid,
                          }}
                        />
                      </>
                    )}
                  </Layout>
                );
              }}
            </Formik>
          </Box>
        </Tabs>
      </Page>
    </>
  );
}
