import React from "react";
import styled from "styled-components";
import BasePhoneInput from "react-phone-number-input";

const PhoneInputWrapper = styled.div`
  input {
    font-size: 1.4rem !important;
    line-height: 2.4rem;
    text-transform: initial;
    border-color: #637381;
    border-size: 1px;
    min-height: 3.6rem;
    margin: 0;
    padding: 0.5rem 1.2rem;
    border: 0.1rem solid var(--p-border, #c4cdd5);
    border-radius: var(--p-border-radius-base, 3px);
  }
  .label-wrapper {
    margin-bottom: 0.4rem;
    align-items: baseline;
  }
  .help-label {
    margin-top: 0.4rem;
    color: var(--p-text-subdued, #637381);
  }
  .error-label {
    margin-top: 0.4rem;
    color: var(--p-text-critical, #bf0711);
  }
  .PhoneWarning {
    input {
      border-color: #bf0711;
      background-color: #fbeae5;
      box-shadow: inset 0 1px 0 0 rgba(99, 115, 129, 0.05);
    }
  }
`;

export default function PhoneInput({ phone, phoneWarning, onBlur, onChange }) {
  return (
    <PhoneInputWrapper>
      <div className="label-wrapper">
        <label>Phone Number</label>
      </div>
      <BasePhoneInput
        id="phone"
        className={
          phoneWarning ? "PhoneInputInput PhoneWarning" : "PhoneInputInput"
        }
        // placeholder="Enter phone number"
        defaultCountry="US"
        value={phone}
        onChange={onChange}
        onBlur={onBlur}
      />
      {phoneWarning && <div className="error-label">{phoneWarning}</div>}
      <div className="help-label">
        We’ll use this phone number to send the ID check instructions to.
      </div>
    </PhoneInputWrapper>
  );
}
