import React from "react";
import {
  Layout,
  Card,
  TextContainer,
  FormLayout,
  TextField,
  TextStyle,
} from "@shopify/polaris";
import get from "lodash/get";
import without from "lodash/without";

export default function IdCheckNotificationSettings({
  values,
  setFieldValue,
  errors,
}) {
  return (
    <Layout.AnnotatedSection
      title="ID Check Notification Settings"
      description={
        <>
          <p>
            These emails will receive notifications when customers complete ID
            checks.
          </p>
          <br />
          <p>
            Add your team members emails to keep them updated when customers
            complete ID checks.
          </p>
        </>
      }
    >
      <Card
        fullWidth
        title="Email Notifications"
        primaryFooterAction={{
          content: "Add email",
          onAction: () => {
            setFieldValue("contactEmails", [...values.contactEmails, ""]);
          },
        }}
      >
        <Card.Section>
          <TextContainer>
            <TextStyle variation="subdued">
              When an ID check is completed, an emails will be will be sent to
              these email addresses
            </TextStyle>
          </TextContainer>
          {values.contactEmails.map((contactEmail, i) => (
            <FormLayout key={i}>
              <TextField
                label="Email"
                type="email"
                error={get(errors, `contactEmails[${i}]`)}
                value={contactEmail}
                onChange={(value) => {
                  setFieldValue(`contactEmails[${i}]`, value);
                }}
                clearButton={i != 0}
                onClearButtonClick={(email) => {
                  const newContactEmails = without(
                    values.contactEmails,
                    values.contactEmails[i]
                  );
                  console.log(newContactEmails);
                  setFieldValue("contactEmails", newContactEmails);
                }}
              />
            </FormLayout>
          ))}
        </Card.Section>
      </Card>
    </Layout.AnnotatedSection>
  );
}
