import axios from "axios";
import combineUrls from "axios/lib/helpers/combineURLs";

axios.interceptors.request.use((config) => {
  const nonce = window.realIdApiSettings.nonce;
  config.headers["X-WP-Nonce"] = nonce;
  config.headers["Content-Type"] = "application/json";

  return config;
});

/**
 * Get the root wp-json URL
 *
 * @returns String
 */
function root() {
  return window.realIdApiSettings.root;
}

/**
 * get the full WP API route for the given path
 *
 * @param {String} path
 * @returns
 */
function wpRoute(path) {
  return combineUrls(root(), path);
}

export const getCurrentShop = () => {
  return axios.get(wpRoute(`/real-id/v1/shop`));
};

export const getChecks = (params) => {
  return axios.get(wpRoute("/real-id/v1/checks"), {
    params,
  });
};

export const createCheck = (data) => {
  return axios.post(wpRoute("/real-id/v1/checks"), data);
};

export const getCheck = (checkId) => {
  return axios.get(wpRoute(`/real-id/v1/checks/${checkId}`));
};

export const deleteCheckData = (checkId) => {
  return axios.delete(wpRoute(`/real-id/v1/checks/${checkId}/data`));
};

export const deliverCheckReminder = (checkId, data) => {
  return axios.post(wpRoute(`/real-id/v1/checks/${checkId}/reminder`), data);
};

export const manuallyApproveCheck = (checkId) => {
  return axios.post(wpRoute(`/real-id/v1/checks/${checkId}/approve`));
};

export const manuallyRejectCheck = (checkId) => {
  return axios.post(wpRoute(`/real-id/v1/checks/${checkId}/reject`));
};

export const updateSettings = (settings) => {
  return axios.post(wpRoute(`/real-id/v1/shop/settings`), settings);
};

export const deactivateLicense = () => {
  return axios.delete(wpRoute("/real-id/v1/license/deactivate"));
};

export const activateLicense = ({ licenseKey }) => {
  return axios.post(wpRoute("/real-id/v1/license/activate"), {
    licenseKey,
  });
};

export const updateDeliveryMethods = (params) => {
  return axios.post(
    wpRoute("/real-id/v1/shop/update-delivery-methods"),
    params
  );
};

export const addEmailSignature = (params) => {
  return axios.post(wpRoute("/real-id/v1/shop/email-signature/add"), params);
};

export const checkEmailSignatureVerificationStatus = () => {
  return axios.post(
    wpRoute("/real-id/v1/shop/email-signature/check-verification-status")
  );
};

export const removeEmailSignature = (params) => {
  return axios.delete(
    wpRoute("/real-id/v1/shop/email-signature/remove"),
    params
  );
};

export const resetToSandboxMode = () => {
  return axios.post(wpRoute("/real-id/v1/shop/reset"));
};

export const listCategories = ({ currentPage = 1 }) => {
  return axios.get(wpRoute("/wc/v3/products/categories"), {
    params: {
      page: currentPage,
      per_page: 5,
    },
  });
};

export const getCurrentLicenseKey = () => {
  return axios.get(wpRoute("/real-id/v1/license-key"));
};
